@@ -31,14 +31,14 @@ namespace KMime
 namespace Headers
 {
 
+// Note that this entire class hierarchy has no virtual dtor, in order to not
+// have a second set of vtables, as this is a rather high-volume class.
+// This means it's not enough to just delete d_ptr in the base class, but
+// we need to delete it with the exact sub-class it was created.
+
 class BasePrivate
 {
 public:
-    BasePrivate() : parent(0) {}
-
-    virtual ~BasePrivate() {}
-
-    KMime::Content *parent;
     QByteArray encCS;
 };
 
@@ -112,6 +112,11 @@ public:
 class MailCopiesToPrivate : public Generics::AddressListPrivate
 {
 public:
+    MailCopiesToPrivate() :
+        Generics::AddressListPrivate(),
+        alwaysCopy(false),
+        neverCopy(false)
+    {}
     bool alwaysCopy;
     bool neverCopy;
 };
@@ -119,6 +124,11 @@ public:
 class ContentTransferEncodingPrivate : public Generics::TokenPrivate
 {
 public:
+    ContentTransferEncodingPrivate() :
+        Generics::TokenPrivate(),
+        cte(CE7Bit),
+        decoded(true)
+    {}
     contentEncoding cte;
     bool decoded;
 };
@@ -126,6 +136,10 @@ public:
 class ContentTypePrivate : public Generics::ParametrizedPrivate
 {
 public:
+    ContentTypePrivate() :
+        Generics::ParametrizedPrivate(),
+        category(CCsingle)
+    {}
     QByteArray mimeType;
     contentCategory category;
 };
@@ -133,13 +147,20 @@ public:
 class ContentDispositionPrivate : public Generics::ParametrizedPrivate
 {
 public:
+    ContentDispositionPrivate() :
+        Generics::ParametrizedPrivate(),
+        disposition(CDInvalid)
+    {}
     contentDisposition disposition;
 };
 
 class GenericPrivate : public Generics::UnstructuredPrivate
 {
 public:
-    GenericPrivate() : type(0) {}
+    GenericPrivate() :
+        Generics::UnstructuredPrivate(),
+        type(0)
+    {}
     ~GenericPrivate()
     {
         delete[] type;
@@ -164,12 +185,16 @@ public:
 class NewsgroupsPrivate : public Generics::StructuredPrivate
 {
 public:
-    QList<QByteArray> groups;
+    QVector<QByteArray> groups;
 };
 
 class LinesPrivate : public Generics::StructuredPrivate
 {
 public:
+    LinesPrivate() :
+        Generics::StructuredPrivate(),
+        lines(-1)
+    {}
     int lines;
 };
 
